from starhtml import FT, A, Icon, Li, Nav, Ol, Span

from .utils import cn


def Breadcrumb(*children, class_name: str = "", cls: str = "", **attrs) -> FT:
    return Nav(
        *children,
        aria_label="breadcrumb",
        data_slot="breadcrumb",
        cls=cn("", class_name, cls),
        **attrs,
    )


def BreadcrumbList(*children, class_name: str = "", cls: str = "", **attrs) -> FT:
    return Ol(
        *children,
        data_slot="breadcrumb-list",
        cls=cn(
            "text-muted-foreground flex flex-wrap items-center gap-1.5 text-sm break-words sm:gap-2.5",
            class_name,
            cls,
        ),
        **attrs,
    )


def BreadcrumbItem(*children, class_name: str = "", cls: str = "", **attrs) -> FT:
    return Li(
        *children,
        data_slot="breadcrumb-item",
        cls=cn("inline-flex items-center gap-1.5", class_name, cls),
        **attrs,
    )


def BreadcrumbLink(
    *children, href: str = "#", class_name: str = "", cls: str = "", **attrs
) -> FT:
    return A(
        *children,
        href=href,
        data_slot="breadcrumb-link",
        cls=cn("hover:text-foreground transition-colors", class_name, cls),
        **attrs,
    )


def BreadcrumbPage(*children, class_name: str = "", cls: str = "", **attrs) -> FT:
    return Span(
        *children,
        role="link",
        aria_disabled="true",
        aria_current="page",
        data_slot="breadcrumb-page",
        cls=cn("text-foreground font-normal", class_name, cls),
        **attrs,
    )


def BreadcrumbSeparator(*children, class_name: str = "", cls: str = "", **attrs) -> FT:
    separator_content = children if children else (Icon("lucide:chevron-right"),)

    return Li(
        *separator_content,
        role="presentation",
        aria_hidden="true",
        data_slot="breadcrumb-separator",
        cls=cn("[&>svg]:size-3.5", class_name, cls),
        **attrs,
    )


def BreadcrumbEllipsis(class_name: str = "", cls: str = "", **attrs) -> FT:
    return Span(
        Icon("lucide:more-horizontal", cls="size-4"),
        Span("More", cls="sr-only"),
        role="presentation",
        aria_hidden="true",
        data_slot="breadcrumb-ellipsis",
        cls=cn("flex size-9 items-center justify-center", class_name, cls),
        **attrs,
    )



from typing import Literal

from starhtml import FT, H1, H2, H3, H4, H5, H6, Div, P

from .utils import cn

HeadingLevel = Literal["h1", "h2", "h3", "h4", "h5", "h6"]


def Card(
    *children,
    class_name: str = "",
    cls: str = "",
    **attrs,
) -> FT:
    classes = cn(
        "bg-card text-card-foreground flex flex-col gap-6 rounded-xl border py-6 shadow-sm",
        class_name,
        cls,
    )
    return Div(*children, cls=classes, data_slot="card", **attrs)


def CardHeader(
    *children,
    class_name: str = "",
    cls: str = "",
    **attrs,
) -> FT:
    classes = cn(
        "@container/card-header grid auto-rows-min grid-rows-[auto_auto] items-start gap-1.5 px-6 has-data-[slot=card-action]:grid-cols-[1fr_auto] [.border-b]:pb-6",
        class_name,
        cls,
    )
    return Div(*children, cls=classes, data_slot="card-header", **attrs)


def CardTitle(
    *children,
    level: HeadingLevel = "h3",
    class_name: str = "",
    cls: str = "",
    **attrs,
) -> FT:
    classes = cn("leading-none font-semibold", class_name, cls)

    heading_components = {
        "h1": H1,
        "h2": H2,
        "h3": H3,
        "h4": H4,
        "h5": H5,
        "h6": H6,
    }

    Heading = heading_components[level]
    return Heading(*children, cls=classes, data_slot="card-title", **attrs)


def CardDescription(
    *children,
    class_name: str = "",
    cls: str = "",
    **attrs,
) -> FT:
    classes = cn("text-muted-foreground text-sm", class_name, cls)
    return P(*children, cls=classes, data_slot="card-description", **attrs)


def CardAction(
    *children,
    class_name: str = "",
    cls: str = "",
    **attrs,
) -> FT:
    classes = cn(
        "col-start-2 row-span-2 row-start-1 self-start justify-self-end",
        class_name,
        cls,
    )
    return Div(*children, cls=classes, data_slot="card-action", **attrs)


def CardContent(
    *children,
    class_name: str = "",
    cls: str = "",
    **attrs,
) -> FT:
    classes = cn("px-6", class_name, cls)
    return Div(*children, cls=classes, data_slot="card-content", **attrs)


def CardFooter(
    *children,
    class_name: str = "",
    cls: str = "",
    **attrs,
) -> FT:
    classes = cn("flex items-center px-6 [.border-t]:pt-6", class_name, cls)
    return Div(*children, cls=classes, data_slot="card-footer", **attrs)



from itertools import count
from typing import Any
from uuid import uuid4

from starhtml import FT, Div, Label, P, Span
from starhtml import Input as HTMLInput
from starhtml.datastar import ds_class, ds_on_change, ds_signals, value

from .utils import cn, inject_signals, make_injectable

_radio_group_ids = count(1)


def RadioGroup(
    *children: Any,
    initial_value: str | None = None,
    signal: str = "",
    disabled: bool = False,
    required: bool = False,
    class_name: str = "",
    cls: str = "",
    **attrs: Any,
) -> FT:
    signal = signal or f"radio_{next(_radio_group_ids)}"
    group_name = f"radio_group_{signal}"

    processed_children = inject_signals(children, signal, group_name, initial_value)

    return Div(
        *processed_children,
        ds_signals({signal: value(initial_value or "")}),
        cls=cn("grid gap-2", class_name, cls),
        data_slot="radio-group",
        data_radio_signal=signal,
        data_radio_name=group_name,
        role="radiogroup",
        aria_required="true" if required else None,
        **attrs,
    )


def RadioGroupItem(
    value: str,
    label: str | None = None,
    disabled: bool = False,
    class_name: str = "",
    cls: str = "",
    indicator_cls: str = "",
    **attrs: Any,
) -> FT:
    def _inject_signal(signal, group_name, default_value=None):
        radio_id = f"radio_{str(uuid4())[:8]}"
        filtered_attrs = {k: v for k, v in attrs.items() if k != "name"}

        radio_input = HTMLInput(
            ds_on_change(f"${signal} = '{value}'"),
            type="radio",
            id=radio_id,
            value=value,
            name=group_name,
            disabled=disabled,
            data_slot="radio-input",
            cls="sr-only peer",
            **filtered_attrs,
        )

        visual_radio = Div(
            Div(
                Div(cls="size-2 rounded-full bg-primary"),
                ds_class(
                    opacity_100=f"${signal} === '{value}'",
                    opacity_0=f"${signal} !== '{value}'",
                ),
                cls=cn(
                    "absolute inset-0 flex items-center justify-center",
                    indicator_cls,
                ),
                data_slot="radio-indicator",
            ),
            cls=cn(
                "relative aspect-square size-4 shrink-0 rounded-full border transition-all",
                "border-input bg-background dark:bg-input/30",
                "peer-disabled:cursor-not-allowed peer-disabled:opacity-50",
                class_name,
                cls,
            ),
            data_slot="radio-visual",
        )

        if not label:
            return Div(
                radio_input,
                visual_radio,
                cls="relative inline-flex items-center",
                data_slot="radio-container",
            )

        return Label(
            radio_input,
            visual_radio,
            Span(
                label,
                cls="text-sm font-medium leading-none peer-disabled:cursor-not-allowed peer-disabled:opacity-50",
            ),
            for_=radio_id,
            cls="flex items-center gap-2 cursor-pointer",
            data_slot="radio-container",
        )

    return make_injectable(_inject_signal)


def RadioGroupWithLabel(
    label: str,
    options: list[dict[str, Any]],
    value: str | None = None,
    signal: str | None = None,
    name: str | None = None,
    helper_text: str | None = None,
    error_text: str | None = None,
    disabled: bool = False,
    required: bool = False,
    orientation: str = "vertical",
    class_name: str = "",
    cls: str = "",
    **attrs: Any,
) -> FT:
    base_id = str(uuid4())[:8]
    signal = signal or f"radio_{base_id}"
    name = name or f"radio_group_{signal}"
    group_id = f"radiogroup_{base_id}"

    radio_group_classes = cn(
        "flex gap-2",
        "flex-col" if orientation == "vertical" else "flex-row gap-6",
    )

    return Div(
        label
        and Label(
            label,
            required and Span(" *", cls="text-destructive") or None,
            cls="text-sm font-medium mb-3 block",
            for_=group_id,
        )
        or None,
        RadioGroup(
            *[
                RadioGroupItem(
                    value=option["value"],
                    label=option.get("label"),
                    disabled=disabled or option.get("disabled", False),
                )
                for option in options
            ],
            value=value,
            signal=signal,
            name=name,
            disabled=disabled,
            required=required,
            cls=radio_group_classes,
            id=group_id,
            aria_invalid="true" if error_text else None,
        ),
        error_text and P(error_text, cls="text-sm text-destructive mt-1.5") or None,
        helper_text
        and not error_text
        and P(helper_text, cls="text-sm text-muted-foreground mt-1.5")
        or None,
        cls=cn("space-y-1.5", class_name, cls),
        **attrs,
    )
